@@ -68,7 +68,10 @@ bool File::Open(const wchar *Name,uint Mode)
 
     wchar LongName[NM];
     if (GetWinLongPath(Name,LongName,ASIZE(LongName)))
+    {
       hNewFile=CreateFile(LongName,Access,ShareMode,NULL,OPEN_EXISTING,Flags,NULL);
+      LastError=GetLastError();
+    }
   }
 
   if (hNewFile==BAD_HANDLE && LastError==ERROR_FILE_NOT_FOUND)
@@ -188,23 +191,25 @@ bool File::WCreate(const wchar *Name,uint Mode)
 bool File::Close()
 {
   bool Success=true;
-  if (HandleType!=FILE_HANDLENORMAL)
-    HandleType=FILE_HANDLENORMAL;
-  else
-    if (hFile!=BAD_HANDLE)
+
+  if (hFile!=BAD_HANDLE)
+  {
+    if (!SkipClose)
     {
-      if (!SkipClose)
-      {
 #ifdef _WIN_ALL
+      // We use the standard system handle for stdout in Windows
+      // and it must not  be closed here.
+      if (HandleType==FILE_HANDLENORMAL)
         Success=CloseHandle(hFile)==TRUE;
 #else
-        Success=fclose(hFile)!=EOF;
+      Success=fclose(hFile)!=EOF;
 #endif
-      }
-      hFile=BAD_HANDLE;
-      if (!Success && AllowExceptions)
-        ErrHandler.CloseError(FileName);
     }
+    hFile=BAD_HANDLE;
+  }
+  HandleType=FILE_HANDLENORMAL;
+  if (!Success && AllowExceptions)
+    ErrHandler.CloseError(FileName);
   return Success;
 }
 
@@ -250,26 +255,16 @@ void File::Write(const void *Data,size_t Size)
 {
   if (Size==0)
     return;
-#ifndef _WIN_CE
-  if (HandleType!=FILE_HANDLENORMAL)
-    switch(HandleType)
-    {
-      case FILE_HANDLESTD:
+  if (HandleType==FILE_HANDLESTD)
+  {
 #ifdef _WIN_ALL
-        hFile=GetStdHandle(STD_OUTPUT_HANDLE);
+    hFile=GetStdHandle(STD_OUTPUT_HANDLE);
 #else
-        hFile=stdout;
-#endif
-        break;
-      case FILE_HANDLEERR:
-#ifdef _WIN_ALL
-        hFile=GetStdHandle(STD_ERROR_HANDLE);
-#else
-        hFile=stderr;
-#endif
-        break;
-    }
+    // Cannot use the standard stdout here, because it already has wide orientation.
+    if (hFile==BAD_HANDLE)
+      hFile=fdopen(dup(1),"w"); // Open new stdout stream.
 #endif
+  }
   while (1)
   {
     bool Success=false;
@@ -364,7 +359,6 @@ int File::DirectRead(void *Data,size_t Size)
   const size_t MaxDeviceRead=20000;
   const size_t MaxLockedRead=32768;
 #endif
-#ifndef _WIN_CE
   if (HandleType==FILE_HANDLESTD)
   {
 #ifdef _WIN_ALL
@@ -375,7 +369,6 @@ int File::DirectRead(void *Data,size_t Size)
     hFile=stdin;
 #endif
   }
-#endif
 #ifdef _WIN_ALL
   DWORD Read;
   if (!ReadFile(hFile,Data,(DWORD)Size,&Read,NULL))
